@@ -1,5 +1,4 @@
 use super::*;
-use crate::utils::should_deny_url;
 use crate::{
     client,
     event_handlers::{
@@ -13,11 +12,11 @@ use crate::{
         StatField::{LinksExtracted, TotalExpected},
     },
     url::FeroxUrl,
-    utils::{logged_request, make_request},
-    DEFAULT_METHOD,
+    utils::{logged_request, make_request, should_deny_url},
+    ExtractionResult, DEFAULT_METHOD,
 };
 use anyhow::{bail, Context, Result};
-use reqwest::{StatusCode, Url};
+use reqwest::{Client, StatusCode, Url};
 use scraper::{Html, Selector};
 use std::collections::HashSet;
 use tokio::sync::oneshot;
@@ -57,19 +56,77 @@ pub struct Extractor<'a> {
 /// Extractor implementation
 impl<'a> Extractor<'a> {
     /// perform extraction from the given target and return any links found
-    pub async fn extract(&self) -> Result<(HashSet<String>, bool)> {
-        log::trace!("enter: extract (this fn has associated trace exit msg)");
+    pub async fn extract(&self) -> Result<ExtractionResult> {
+        log::trace!(
+            "enter: extract({:?}) (this fn has no associated trace exit msg)",
+            self.target
+        );
         match self.target {
             ExtractionTarget::ResponseBody => Ok(self.extract_from_body().await?),
             ExtractionTarget::RobotsTxt => Ok(self.extract_from_robots().await?),
-            ExtractionTarget::ParseHtml => Ok(self.parse_html().await?),
+            ExtractionTarget::DirectoryListing => Ok(self.extract_from_dir_listing().await?),
         }
     }
 
+    /// wrapper around logic that performs the following:
+    /// - parses `url_to_parse`
+    /// - bails if the parsed url doesn't belong to the original host/domain
+    /// - otherwise, calls `add_all_sub_paths` with the parsed result
+    fn parse_url_and_add_subpaths(
+        &self,
+        url_to_parse: &str,
+        original_url: &Url,
+        links: &mut HashSet<String>,
+    ) -> Result<()> {
+        log::trace!("enter: parse_url_and_add_subpaths({:?})", links);
+
+        match Url::parse(url_to_parse) {
+            Ok(absolute) => {
+                if absolute.domain() != original_url.domain()
+                    || absolute.host() != original_url.host()
+                {
+                    // domains/ips are not the same, don't scan things that aren't part of the original
+                    // target url
+                    bail!("parsed url does not belong to original domain/host");
+                }
+
+                if self.add_all_sub_paths(absolute.path(), links).is_err() {
+                    log::warn!("could not add sub-paths from {} to {:?}", absolute, links);
+                }
+            }
+            Err(e) => {
+                // this is the expected error that happens when we try to parse a url fragment
+                //     ex: Url::parse("/login") -> Err("relative URL without a base")
+                // while this is technically an error, these are good results for us
+                if e.to_string().contains("relative URL without a base") {
+                    if self.add_all_sub_paths(url_to_parse, links).is_err() {
+                        log::warn!(
+                            "could not add sub-paths from {} to {:?}",
+                            url_to_parse,
+                            links
+                        );
+                    }
+                } else {
+                    // unexpected error has occurred
+                    log::warn!("Could not parse given url: {}", e);
+                    self.handles.stats.send(AddError(Other)).unwrap_or_default();
+                }
+            }
+        }
+
+        log::trace!("exit: parse_url_and_add_subpaths");
+        Ok(())
+    }
+
     /// given a set of links from a normal http body response, task the request handler to make
     /// the requests
-    pub async fn request_links(&self, links: HashSet<String>) -> Result<()> {
+    pub async fn request_links(&mut self, links: HashSet<String>) -> Result<()> {
         log::trace!("enter: request_links({:?})", links);
+
+        if links.is_empty() {
+            return Ok(());
+        }
+
         let recursive = if self.handles.config.no_recursion {
             RecursionStatus::NotRecursive
         } else {
@@ -77,6 +134,7 @@ impl<'a> Extractor<'a> {
         };
 
         let scanned_urls = self.handles.ferox_scans()?;
+        self.update_stats(links.len())?;
 
         for link in links {
             let mut resp = match self.request_link(&link).await {
@@ -100,6 +158,10 @@ impl<'a> Extractor<'a> {
 
                 scanned_urls.add_file_scan(resp.url().as_str(), ScanOrder::Latest);
 
+                if self.handles.config.collect_extensions {
+                    resp.parse_extension(self.handles.clone())?;
+                }
+
                 if let Err(e) = resp.send_report(self.handles.output.tx.clone()) {
                     log::warn!("Could not send FeroxResponse to output handler: {}", e);
                 }
@@ -135,8 +197,26 @@ impl<'a> Extractor<'a> {
         Ok(())
     }
 
-    /// Given a `reqwest::Response`, perform the following actions
-    ///   - parse the response's text for links using the linkfinder regex
+    /// wrapper around link extraction via html attributes
+    fn extract_all_links_from_html_tags(
+        &self,
+        resp_url: &Url,
+        links: &mut HashSet<String>,
+        html: &Html,
+    ) {
+        self.extract_links_by_attr(resp_url, links, html, "a", "href");
+        self.extract_links_by_attr(resp_url, links, html, "img", "src");
+        self.extract_links_by_attr(resp_url, links, html, "form", "action");
+        self.extract_links_by_attr(resp_url, links, html, "script", "src");
+        self.extract_links_by_attr(resp_url, links, html, "iframe", "src");
+        self.extract_links_by_attr(resp_url, links, html, "div", "src");
+        self.extract_links_by_attr(resp_url, links, html, "frame", "src");
+        self.extract_links_by_attr(resp_url, links, html, "embed", "src");
+        self.extract_links_by_attr(resp_url, links, html, "script", "src");
+    }
+
+    /// Given the body of a `reqwest::Response`, perform the following actions
+    ///   - parse the body for links using the linkfinder regex
     ///   - for every link found take its url path and parse each sub-path
     ///     - example: Response contains a link fragment `homepage/assets/img/icons/handshake.svg`
     ///       with a base url of http://localhost, the following urls would be returned:
@@ -145,69 +225,32 @@ impl<'a> Extractor<'a> {
     ///         - homepage/assets/img/
     ///         - homepage/assets/
     ///         - homepage/
-    pub(super) async fn extract_from_body(&self) -> Result<(HashSet<String>, bool)> {
-        log::trace!("enter: extract_from_body");
+    fn extract_all_links_from_javascript(
+        &self,
+        response_body: &str,
+        response_url: &Url,
+        links: &mut HashSet<String>,
+    ) {
+        log::trace!(
+            "enter: extract_all_links_from_javascript(html body..., {}, {:?})",
+            response_url.as_str(),
+            links
+        );
 
-        let mut links = HashSet::<String>::new();
-        let dirlist_flag = false;
-
-        // Response
-        let response = self.response.unwrap();
-        let resp_url = response.url();
-        let body = response.text();
-        let html = Html::parse_document(body);
-
-        // Extract Links
-        self.extract_links_by_attr(resp_url, &mut links, &html, "a", "href");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "img", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "form", "action");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "script", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "iframe", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "div", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "frame", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "embed", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "script", "src");
-
-        for capture in self.links_regex.captures_iter(body) {
+        for capture in self.links_regex.captures_iter(response_body) {
             // remove single & double quotes from both ends of the capture
             // capture[0] is the entire match, additional capture groups start at [1]
             let link = capture[0].trim_matches(|c| c == '\'' || c == '"');
 
-            match Url::parse(link) {
-                Ok(absolute) => {
-                    if absolute.domain() != self.response.unwrap().url().domain()
-                        || absolute.host() != self.response.unwrap().url().host()
-                    {
-                        // domains/ips are not the same, don't scan things that aren't part of the original
-                        // target url
-                        continue;
-                    }
-
-                    if self.add_all_sub_paths(absolute.path(), &mut links).is_err() {
-                        log::warn!("could not add sub-paths from {} to {:?}", absolute, links);
-                    }
-                }
-                Err(e) => {
-                    // this is the expected error that happens when we try to parse a url fragment
-                    //     ex: Url::parse("/login") -> Err("relative URL without a base")
-                    // while this is technically an error, these are good results for us
-                    if e.to_string().contains("relative URL without a base") {
-                        if self.add_all_sub_paths(link, &mut links).is_err() {
-                            log::warn!("could not add sub-paths from {} to {:?}", link, links);
-                        }
-                    } else {
-                        // unexpected error has occurred
-                        log::warn!("Could not parse given url: {}", e);
-                        self.handles.stats.send(AddError(Other)).unwrap_or_default();
-                    }
-                }
+            if self
+                .parse_url_and_add_subpaths(link, response_url, links)
+                .is_err()
+            {
+                // purposely not logging the error here, due to the frequency with which it gets hit
             }
         }
 
-        self.update_stats(links.len())?;
-
-        log::trace!("exit: extract_from_body -> {:?} {}", links, dirlist_flag);
-        Ok((links, dirlist_flag))
+        log::trace!("exit: extract_all_links_from_javascript");
     }
 
     /// take a url fragment like homepage/assets/img/icons/handshake.svg and
@@ -227,6 +270,45 @@ impl<'a> Extractor<'a> {
         Ok(())
     }
 
+    /// given a url path, trim whitespace, remove slashes, and queries/fragments; return the
+    /// normalized string
+    pub(super) fn normalize_url_path(&self, path: &str) -> String {
+        log::trace!("enter: normalize_url_path({})", path);
+
+        // remove whitespace and leading '/'
+        let path_str: String = path
+            .trim()
+            .trim_start_matches('/')
+            .chars()
+            .filter(|char| !char.is_whitespace())
+            .collect();
+
+        // snippets from rfc-3986:
+        //
+        //          foo://example.com:8042/over/there?name=ferret#nose
+        //          \_/   \______________/\_________/ \_________/ \__/
+        //           |           |            |            |        |
+        //        scheme     authority       path        query   fragment
+        //
+        // The path component is terminated
+        //    by the first question mark ("?") or number sign ("#") character, or
+        //    by the end of the URI.
+        //
+        // The query component is indicated by the first question
+        //    mark ("?") character and terminated by a number sign ("#") character
+        //    or by the end of the URI.
+        let (path_str, _discarded) = path_str
+            .split_once('?')
+            // if there isn't a '?', try to remove a fragment
+            .unwrap_or_else(|| {
+                // if there isn't a '#', return (original, empty)
+                path_str.split_once('#').unwrap_or((&path_str, ""))
+            });
+
+        log::trace!("exit: normalize_url_path -> {}", path_str);
+        path_str.into()
+    }
+
     /// Iterate over a given path, return a list of every sub-path found
     ///
     /// example: `path` contains a link fragment `homepage/assets/img/icons/handshake.svg`
@@ -240,8 +322,13 @@ impl<'a> Extractor<'a> {
         log::trace!("enter: get_sub_paths_from_path({})", path);
         let mut paths = vec![];
 
+        let normalized_path = self.normalize_url_path(path);
+
         // filter out any empty strings caused by .split
-        let mut parts: Vec<&str> = path.split('/').filter(|s| !s.is_empty()).collect();
+        let mut parts: Vec<&str> = normalized_path
+            .split('/')
+            .filter(|s| !s.is_empty())
+            .collect();
 
         let length = parts.len();
 
@@ -274,7 +361,7 @@ impl<'a> Extractor<'a> {
         paths
     }
 
-    /// simple helper to stay DRY, trys to join a url + fragment and add it to the `links` HashSet
+    /// simple helper to stay DRY, tries to join a url + fragment and add it to the `links` HashSet
     pub(super) fn add_link_to_set_of_links(
         &self,
         link: &str,
@@ -283,15 +370,15 @@ impl<'a> Extractor<'a> {
         log::trace!("enter: add_link_to_set_of_links({}, {:?})", link, links);
 
         let old_url = match self.target {
-            ExtractionTarget::ResponseBody => self.response.unwrap().url().clone(),
-            ExtractionTarget::ParseHtml | ExtractionTarget::RobotsTxt => {
-                match Url::parse(&self.url) {
-                    Ok(u) => u,
-                    Err(e) => {
-                        bail!("Could not parse {}: {}", self.url, e);
-                    }
-                }
+            ExtractionTarget::ResponseBody | ExtractionTarget::DirectoryListing => {
+                self.response.unwrap().url().clone()
             }
+            ExtractionTarget::RobotsTxt => match Url::parse(&self.url) {
+                Ok(u) => u,
+                Err(e) => {
+                    bail!("Could not parse {}: {}", self.url, e);
+                }
+            },
         };
 
         let new_url = old_url
@@ -346,7 +433,6 @@ impl<'a> Extractor<'a> {
             new_response,
             url,
             DEFAULT_METHOD,
-            true,
             self.handles.config.output_level,
         )
         .await;
@@ -364,11 +450,10 @@ impl<'a> Extractor<'a> {
     ///     http://localhost/stuff/things
     /// this function requests:
     ///     http://localhost/robots.txt
-    pub(super) async fn extract_from_robots(&self) -> Result<(HashSet<String>, bool)> {
+    pub(super) async fn extract_from_robots(&self) -> Result<ExtractionResult> {
         log::trace!("enter: extract_robots_txt");
 
-        let mut links: HashSet<String> = HashSet::new();
-        let dirlist_flag = false;
+        let mut result: HashSet<_> = ExtractionResult::new();
 
         // request
         let response = self.make_extract_request("/robots.txt").await?;
@@ -377,73 +462,60 @@ impl<'a> Extractor<'a> {
         for capture in self.robots_regex.captures_iter(body) {
             if let Some(new_path) = capture.name("url_path") {
                 let mut new_url = Url::parse(&self.url)?;
+
                 new_url.set_path(new_path.as_str());
-                if self.add_all_sub_paths(new_url.path(), &mut links).is_err() {
-                    log::warn!("could not add sub-paths from {} to {:?}", new_url, links);
+
+                if self.add_all_sub_paths(new_url.path(), &mut result).is_err() {
+                    log::warn!("could not add sub-paths from {} to {:?}", new_url, result);
                 }
             }
         }
 
-        self.update_stats(links.len())?;
-
-        log::trace!("exit: extract_robots_txt -> {:?} {}", links, dirlist_flag);
-        Ok((links, dirlist_flag))
+        log::trace!("exit: extract_robots_txt -> {:?}", result);
+        Ok(result)
     }
 
-    /// Entry point to parse html for links (i.e. webscraping, directory listings)
-    /// this function requests:
-    ///     http://localhost/<location>
-    pub(super) async fn parse_html(&self) -> Result<(HashSet<String>, bool)> {
-        log::trace!("enter: parse_html");
+    /// outer-most wrapper for parsing html response bodies in search of additional content.
+    /// performs the following high-level steps:
+    /// - requests the page, if necessary
+    /// - checks the page to see if directory listing is enabled and sucks up all the links, if so
+    /// - uses the linkfinder regex to grab links from embedded javascript/javascript files
+    /// - extracts many different types of link sources from the html itself
+    pub(super) async fn extract_from_body(&self) -> Result<ExtractionResult> {
+        log::trace!("enter: extract_from_body");
 
-        let mut links: HashSet<String> = HashSet::new();
-        let mut dirlist_flag = false;
+        let mut result = ExtractionResult::new();
 
-        // Response
-        let url = Url::parse(&self.url)?;
-        let response = self.make_extract_request(url.path()).await?;
+        let response = self.response.unwrap();
         let resp_url = response.url();
         let body = response.text();
         let html = Html::parse_document(body);
 
-        // Directory listing heuristic detection to not continue scanning
-        // Index of /: apache
-        // Directory Listing for /: tomcat,
-        // Directory Listing -- /: ASP.NET
-        // <host> - /: iis, azure, skipping due to loose heuristic
-        let title_selector = Selector::parse("title").unwrap();
-        for t in html.select(&title_selector) {
-            let title = t.inner_html().to_lowercase();
-            if title.contains("directory listing for /")
-                || title.contains("index of /")
-                || title.contains("directory listing -- /")
-            {
-                log::debug!("Directory listing heuristic detection from \"{}\"", title);
-                dirlist_flag = true;
+        // extract links from html tags/attributes and embedded javascript
+        self.extract_all_links_from_html_tags(resp_url, &mut result, &html);
+        self.extract_all_links_from_javascript(body, resp_url, &mut result);
 
-                self.extract_links_by_attr(resp_url, &mut links, &html, "a", "href");
-                self.update_stats(links.len())?;
+        log::trace!("exit: extract_from_body -> {:?}", result);
+        Ok(result)
+    }
 
-                log::trace!("exit: parse_html -> {:?} {}", links, dirlist_flag);
-                return Ok((links, dirlist_flag));
-            }
-        }
+    /// parses html response bodies in search of <a> tags.
+    ///
+    /// the assumption is that directory listing is turned on and this extraction target simply
+    /// scoops up all the links for the given directory. The test to detect a directory listing
+    /// is located in `HeuristicTests`
+    pub async fn extract_from_dir_listing(&self) -> Result<ExtractionResult> {
+        log::trace!("enter: extract_from_dir_listing");
 
-        // Extract Links
-        self.extract_links_by_attr(resp_url, &mut links, &html, "a", "href");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "img", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "form", "action");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "script", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "iframe", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "div", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "frame", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "embed", "src");
-        self.extract_links_by_attr(resp_url, &mut links, &html, "script", "src");
+        let mut result = ExtractionResult::new();
 
-        self.update_stats(links.len())?;
+        let response = self.response.unwrap();
+        let html = Html::parse_document(response.text());
 
-        log::trace!("exit: parse_html -> {:?} {}", links, dirlist_flag);
-        Ok((links, dirlist_flag))
+        self.extract_links_by_attr(response.url(), &mut result, &html, "a", "href");
+
+        log::trace!("exit: extract_from_dir_listing -> {:?}", result);
+        Ok(result)
     }
 
     /// simple helper to get html links by tag/attribute and add it to the `links` HashSet
@@ -458,41 +530,20 @@ impl<'a> Extractor<'a> {
         log::trace!("enter: extract_links_by_attr");
 
         let selector = Selector::parse(html_tag).unwrap();
+
         let tags = html
             .select(&selector)
             .filter(|a| a.value().attrs().any(|attr| attr.0 == html_attr));
-        for t in tags {
-            if let Some(link) = t.value().attr(html_attr) {
+
+        for tag in tags {
+            if let Some(link) = tag.value().attr(html_attr) {
                 log::debug!("Parsed link \"{}\" from {}", link, resp_url.as_str());
 
-                match Url::parse(link) {
-                    Ok(absolute) => {
-                        if absolute.domain() != resp_url.domain()
-                            || absolute.host() != resp_url.host()
-                        {
-                            // domains/ips are not the same, don't scan things that aren't part of the original
-                            // target url
-                            continue;
-                        }
-
-                        if self.add_all_sub_paths(absolute.path(), links).is_err() {
-                            log::warn!("could not add sub-paths from {} to {:?}", absolute, links);
-                        }
-                    }
-                    Err(e) => {
-                        // this is the expected error that happens when we try to parse a url fragment
-                        //     ex: Url::parse("/login") -> Err("relative URL without a base")
-                        // while this is technically an error, these are good results for us
-                        if e.to_string().contains("relative URL without a base") {
-                            if self.add_all_sub_paths(link, links).is_err() {
-                                log::warn!("could not add sub-paths from {} to {:?}", link, links);
-                            }
-                        } else {
-                            // unexpected error has occurred
-                            log::warn!("Could not parse given url: {}", e);
-                            self.handles.stats.send(AddError(Other)).unwrap_or_default();
-                        }
-                    }
+                if self
+                    .parse_url_and_add_subpaths(link, resp_url, links)
+                    .is_err()
+                {
+                    log::debug!("link didn't belong to the target domain/host: {}", link);
                 }
             }
         }
@@ -507,33 +558,45 @@ impl<'a> Extractor<'a> {
     pub(super) async fn make_extract_request(&self, location: &str) -> Result<FeroxResponse> {
         log::trace!("enter: make_extract_request");
 
-        // more often than not, domain/robots.txt will redirect to www.domain/robots.txt or something
-        // similar; to account for that, create a client that will follow redirects, regardless of
-        // what the user specified for the scanning client. Other than redirects, it will respect
-        // all other user specified settings
-        let follow_redirects = true;
+        // need late binding here to avoid 'creates a temporary which is freed...' in the
+        // `let ... if` below to avoid cloning the client out of config
+        let mut client = Client::new();
 
-        let proxy = if self.handles.config.proxy.is_empty() {
-            None
+        if location == "/robots.txt" {
+            // more often than not, domain/robots.txt will redirect to www.domain/robots.txt or something
+            // similar; to account for that, create a client that will follow redirects, regardless of
+            // what the user specified for the scanning client. Other than redirects, it will respect
+            // all other user specified settings
+            let follow_redirects = true;
+
+            let proxy = if self.handles.config.proxy.is_empty() {
+                None
+            } else {
+                Some(self.handles.config.proxy.as_str())
+            };
+
+            client = client::initialize(
+                self.handles.config.timeout,
+                &self.handles.config.user_agent,
+                follow_redirects,
+                self.handles.config.insecure,
+                &self.handles.config.headers,
+                proxy,
+            )?;
+        }
+
+        let client = if location != "/robots.txt" {
+            &self.handles.config.client
         } else {
-            Some(self.handles.config.proxy.as_str())
+            &client
         };
 
-        let client = client::initialize(
-            self.handles.config.timeout,
-            &self.handles.config.user_agent,
-            follow_redirects,
-            self.handles.config.insecure,
-            &self.handles.config.headers,
-            proxy,
-        )?;
-
         let mut url = Url::parse(&self.url)?;
         url.set_path(location); // overwrite existing path
 
         // purposefully not using logged_request here due to using the special client
         let response = make_request(
-            &client,
+            client,
             &url,
             DEFAULT_METHOD,
             None,
@@ -547,10 +610,10 @@ impl<'a> Extractor<'a> {
             response,
             &self.url,
             DEFAULT_METHOD,
-            true,
             self.handles.config.output_level,
         )
         .await;
+        // note: don't call parse_extension here. If we call it here, it gets called on robots.txt
 
         log::trace!("exit: make_extract_request -> {}", ferox_response);
         Ok(ferox_response)
@@ -558,7 +621,7 @@ impl<'a> Extractor<'a> {
 
     /// update total number of links extracted and expected responses
     fn update_stats(&self, num_links: usize) -> Result<()> {
-        let multiplier = self.handles.config.extensions.len().max(1);
+        let multiplier = self.handles.expected_num_requests_multiplier();
 
         self.handles
             .stats
