@@ -1,9 +1,11 @@
 use std::{
     cmp::max,
-    sync::{atomic::Ordering, Arc, Mutex},
+    collections::HashSet,
+    sync::{self, atomic::Ordering, Arc, Mutex},
 };
 
 use anyhow::Result;
+use lazy_static::lazy_static;
 use leaky_bucket::LeakyBucket;
 use tokio::{
     sync::{oneshot, RwLock},
@@ -18,17 +20,21 @@ use crate::{
         Handles,
     },
     extractor::{ExtractionTarget, ExtractorBuilder},
+    nlp::{Document, TfIdf},
     response::FeroxResponse,
     scan_manager::{FeroxScan, ScanStatus},
     statistics::{StatError::Other, StatField::TotalExpected},
     url::FeroxUrl,
-    utils::logged_request,
+    utils::{logged_request, should_deny_url},
     HIGH_ERROR_RATIO,
 };
 
 use super::{policy_data::PolicyData, FeroxScanner, PolicyTrigger};
-use crate::utils::should_deny_url;
-use std::collections::HashSet;
+
+lazy_static! {
+    /// make sure to note that this is a std rwlock and not tokio
+    pub(crate) static ref TF_IDF: Arc<sync::RwLock<TfIdf>> = Arc::new(sync::RwLock::new(TfIdf::new()));
+}
 
 /// Makes multiple requests based on the presence of extensions
 pub(super) struct Requester {
@@ -303,8 +309,10 @@ impl Requester {
     pub async fn request(&self, word: &str) -> Result<()> {
         log::trace!("enter: request({})", word);
 
-        let urls =
-            FeroxUrl::from_string(&self.target_url, self.handles.clone()).formatted_urls(word)?;
+        let collected = self.handles.collected_extensions();
+
+        let urls = FeroxUrl::from_string(&self.target_url, self.handles.clone())
+            .formatted_urls(word, collected)?;
 
         let should_test_deny = !self.handles.config.url_denylist.is_empty()
             || !self.handles.config.regex_denylist.is_empty();
@@ -331,13 +339,14 @@ impl Requester {
                     continue;
                 }
 
-                let response = logged_request(
-                    &url,
-                    method.as_str(),
-                    Some(self.handles.config.data.as_slice()),
-                    self.handles.clone(),
-                )
-                .await?;
+                let data = if self.handles.config.data.is_empty() {
+                    None
+                } else {
+                    Some(self.handles.config.data.as_slice())
+                };
+
+                let response =
+                    logged_request(&url, method.as_str(), data, self.handles.clone()).await?;
 
                 if (should_tune || self.handles.config.auto_bail)
                     && !atomic_load!(self.policy_data.cooling_down, Ordering::SeqCst)
@@ -361,11 +370,10 @@ impl Requester {
                 }
 
                 // response came back without error, convert it to FeroxResponse
-                let ferox_response = FeroxResponse::from(
+                let mut ferox_response = FeroxResponse::from(
                     response,
                     &self.target_url,
                     method,
-                    true,
                     self.handles.config.output_level,
                 )
                 .await;
@@ -392,20 +400,38 @@ impl Requester {
                     continue;
                 }
 
-                if self.handles.config.extract_links && !ferox_response.status().is_redirection() {
-                    let extractor = ExtractorBuilder::default()
+                if self.handles.config.collect_extensions {
+                    ferox_response.parse_extension(self.handles.clone())?;
+                }
+
+                if self.handles.config.collect_words {
+                    if let Ok(mut guard) = TF_IDF.write() {
+                        let doc = Document::from_html(ferox_response.text());
+                        guard.add_document(doc);
+                        if guard.num_documents() % 12 == 0
+                            || (guard.num_documents() < 5 && guard.num_documents() % 2 == 0)
+                        {
+                            guard.calculate_tf_idf_scores();
+                        }
+                    }
+                }
+
+                if self.handles.config.extract_links {
+                    let mut extractor = ExtractorBuilder::default()
                         .target(ExtractionTarget::ResponseBody)
                         .response(&ferox_response)
                         .handles(self.handles.clone())
                         .build()?;
+
                     let new_links: HashSet<_>;
-                    let extracted = (extractor.extract().await?).0;
+
+                    let result = extractor.extract().await?;
 
                     {
                         // gain and quickly drop the read lock on seen_links, using it while unlocked
                         // to determine if there are any new links to process
                         let read_links = self.seen_links.read().await;
-                        new_links = extracted.difference(&read_links).cloned().collect();
+                        new_links = result.difference(&read_links).cloned().collect();
                     }
 
                     if !new_links.is_empty() {
@@ -417,7 +443,9 @@ impl Requester {
                         }
                     }
 
-                    extractor.request_links(new_links).await?;
+                    if !new_links.is_empty() {
+                        extractor.request_links(new_links).await?;
+                    }
                 }
 
                 // everything else should be reported
@@ -458,12 +486,14 @@ mod tests {
         let (filters_task, filters_handle) = FiltersHandler::initialize();
         let (out_task, out_handle) =
             TermOutHandler::initialize(configuration.clone(), stats_handle.tx.clone());
+        let wordlist = Arc::new(vec![String::from("this_is_a_test")]);
 
         let handles = Arc::new(Handles::new(
             stats_handle,
             filters_handle,
             out_handle,
             configuration.clone(),
+            wordlist,
         ));
 
         let (scan_task, scan_handle) = ScanHandler::initialize(handles.clone());
@@ -587,10 +617,10 @@ mod tests {
 
         let requester = Requester {
             handles,
+            target_url: "http://localhost".to_string(),
             seen_links: RwLock::new(HashSet::<String>::new()),
             tuning_lock: Mutex::new(0),
             ferox_scan: Arc::new(FeroxScan::default()),
-            target_url: "http://localhost".to_string(),
             rate_limiter: RwLock::new(None),
             policy_data: Default::default(),
         };
